@@ -51,7 +51,7 @@ use crate::{vec_resize, vec_truncate};
 mod base64_utils;
 use base64_utils::*;
 mod crc;
-use crc::CRC;
+use crc::Crc;
 
 /// The encoded output stream must be represented in lines of no more
 /// than 76 characters each (see (see [RFC 4880, section
@@ -83,7 +83,7 @@ assert_send_and_sync!(Kind);
 
 #[cfg(test)]
 impl Arbitrary for Kind {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         use self::Kind::*;
         match u8::arbitrary(g) % 5 {
             0 => Message,
@@ -236,7 +236,7 @@ pub struct Writer<W: Write> {
     kind: Kind,
     stash: Vec<u8>,
     column: usize,
-    crc: CRC,
+    crc: Crc,
     header: Vec<u8>,
     dirty: bool,
     scratch: Vec<u8>,
@@ -309,7 +309,7 @@ impl<W: Write> Writer<W> {
             kind,
             stash: Vec::<u8>::with_capacity(2),
             column: 0,
-            crc: CRC::new(),
+            crc: Crc::new(),
             header: Vec::with_capacity(128),
             dirty: false,
             scratch: vec![0; 4096],
@@ -558,7 +558,7 @@ pub struct Reader<'a> {
     kind: Option<Kind>,
     mode: ReaderMode,
     decode_buffer: Vec<u8>,
-    crc: CRC,
+    crc: Crc,
     expect_crc: Option<u32>,
     initialized: bool,
     headers: Vec<(String, String)>,
@@ -593,6 +593,7 @@ impl Default for ReaderMode {
 /// State for transforming a message using the Cleartext Signature
 /// Framework into an inline signed message.
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
+#[allow(clippy::upper_case_acronyms)]
 enum CSFTransformer {
     OPS,
     Literal,
@@ -747,7 +748,7 @@ impl<'a> Reader<'a> {
             kind: None,
             mode,
             decode_buffer: Vec::<u8>::with_capacity(1024),
-            crc: CRC::new(),
+            crc: Crc::new(),
             expect_crc: None,
             headers: Vec::new(),
             initialized: false,
@@ -934,7 +935,7 @@ impl<'a> Reader<'a> {
                 }
 
                 // Possible ASCII-armor header.
-                if let Some((label, len)) = Label::detect_header(&input) {
+                if let Some((label, len)) = Label::detect_header(input) {
                     if label == Label::CleartextSignature && ! self.enable_csft
                     {
                         // We found a message using the Cleartext
@@ -1084,7 +1085,7 @@ impl<'a> Reader<'a> {
             // it couldn't is if we encountered EOF.  We need to strip
             // it.  But, if it ends with \r\n, then we also want to
             // strip the \r too.
-            let line = if let Some(rest) = line.strip_suffix(&"\r\n"[..]) {
+            let line = if let Some(rest) = line.strip_suffix("\r\n") {
                 // \r\n.
                 rest
             } else if let Some(rest) = line.strip_suffix('\n') {
@@ -1422,7 +1423,7 @@ impl<'a> Reader<'a> {
                                             armored data"));
                         }
 
-                        let (dashes, rest) = dash_prefix(&line);
+                        let (dashes, rest) = dash_prefix(line);
                         if dashes.len() > 2 // XXX: heuristic...
                             && rest.starts_with(b"BEGIN PGP SIGNATURE")
                         {
@@ -1450,14 +1451,14 @@ impl<'a> Reader<'a> {
 
                         // First, split off the line ending.
                         let crlf_line_end = line.ends_with(b"\r\n");
-                        line = &line[..line.len()
-                                         - if crlf_line_end { 2 } else { 1 }];
+                        line = &line[..line.len().saturating_sub(
+                            if crlf_line_end { 2 } else { 1 })];
 
                         // Now, trim whitespace off the line.
                         while Some(&b' ') == line.last()
                             || Some(&b'\t') == line.last()
                         {
-                            line = &line[..line.len() - 1];
+                            line = &line[..line.len().saturating_sub(1)];
                         }
 
                         text.extend_from_slice(line);
@@ -2058,12 +2059,12 @@ mod test {
     #[test]
     fn dearmor_yuge() {
         let yuge_key = crate::tests::key("yuge-key-so-yuge-the-yugest.asc");
-        let mut r = Reader::new(Cursor::new(&yuge_key[..]),
+        let mut r = Reader::new(Cursor::new(yuge_key),
                                 ReaderMode::VeryTolerant);
         let mut dearmored = Vec::<u8>::new();
         r.read_to_end(&mut dearmored).unwrap();
 
-        let r = Reader::new(Cursor::new(&yuge_key[..]),
+        let r = Reader::new(Cursor::new(yuge_key),
                             ReaderMode::VeryTolerant);
         let mut dearmored = Vec::<u8>::new();
         for c in r.bytes() {
