@@ -207,6 +207,42 @@ assert_send_and_sync!(KeyBlueprint);
 /// [`UserAttribute`s]: crate::packet::user_attribute::UserAttribute
 /// [`Key`s]: crate::packet::Key
 ///
+/// # Security considerations
+///
+/// ## Expiration
+///
+/// There are two ways to invalidate cryptographic key material:
+/// revocation and liveness.  Both variants come with their own
+/// challenges.  Revocations rely on a robust channel to update
+/// certificates (and attackers may interfere with that).
+///
+/// On the other hand, liveness involves creating key material that
+/// expires after a certain time, then periodically extending the
+/// expiration time.  Again, consumers need a way to update
+/// certificates, but should that fail (maybe because it was
+/// interfered with), the consumer errs on the side of no longer
+/// trusting that key material.
+///
+/// Because of the way metadata is added to OpenPGP certificates,
+/// attackers who control the certificate lookup and update mechanism
+/// may strip components like signatures from the certificate.  This
+/// has implications for the robustness of relying on liveness.
+///
+/// If you first create a certificate that does not expire, and then
+/// change your mind and set an expiration time, an attacker can
+/// simply strip off that update, yielding the original certificate
+/// that does not expire.
+///
+/// Hence, to ensure robust key expiration, you must set an expiration
+/// with [`CertBuilder::set_validity_period`] when you create the
+/// certificate.
+///
+/// By default, the `CertBuilder` creates certificates that do not
+/// expire, because the expiration time is a policy decision and
+/// depends on the use case.  For general purpose certificates,
+/// [`CertBuilder::general_purpose`] sets the validity period to
+/// roughly three years.
+///
 /// # Examples
 ///
 /// Generate a general-purpose certificate with one User ID:
@@ -247,9 +283,11 @@ impl CertBuilder<'_> {
     /// [`CertBuilder::add_signing_subkey`],
     /// [`CertBuilder::add_transport_encryption_subkey`], etc.).
     ///
-    /// [`CertBuilder::add_signing_subkey`]: CertBuilder::add_signing_subkey()
-    /// [`CertBuilder::add_transport_encryption_subkey`]: CertBuilder::add_transport_encryption_subkey()
-    /// [`CertBuilder::add_userid`]: CertBuilder::add_userid()
+    /// By default, the generated certificate does not expire.  It is
+    /// recommended to set a suitable validity period using
+    /// [`CertBuilder::set_validity_period`].  See [this
+    /// section](CertBuilder#expiration) of the type's documentation
+    /// for security considerations of key expiration.
     ///
     /// # Examples
     ///
@@ -297,6 +335,9 @@ impl CertBuilder<'_> {
     /// marked as being appropriate for both data in transit and data
     /// at rest.
     ///
+    /// The certificate and all subkeys are valid for approximately
+    /// three years.
+    ///
     /// # Examples
     ///
     /// ```
@@ -973,8 +1014,8 @@ impl CertBuilder<'_> {
 
     /// Adds a custom subkey.
     ///
-    /// If `expiration` is `None`, the subkey uses the same expiration
-    /// time as the primary key.
+    /// If `validity` is `None`, the subkey will be valid for the same
+    /// period as the primary key.
     ///
     /// Likewise, if `cs` is `None`, the same cipher suite is used as
     /// for the primary key.
@@ -1056,6 +1097,9 @@ impl CertBuilder<'_> {
     ///
     ///   [`SubkeyBinding`]: crate::types::SignatureType::SubkeyBinding
     ///
+    /// If `validity` is `None`, the subkey will be valid for the same
+    /// period as the primary key.
+    ///
     /// # Examples
     ///
     /// This example binds a signing subkey to a certificate,
@@ -1121,7 +1165,8 @@ impl CertBuilder<'_> {
     ///
     /// # Examples
     ///
-    /// Make the primary key certification and signing capable:
+    /// Makes the primary key signing-capable but not
+    /// certification-capable.
     ///
     /// ```
     /// use sequoia_openpgp as openpgp;
@@ -1142,7 +1187,7 @@ impl CertBuilder<'_> {
     /// // Observe that the primary key's certification capability is
     /// // set implicitly.
     /// assert_eq!(cert.with_policy(p, None)?.primary_key().key_flags(),
-    ///            Some(KeyFlags::empty().set_signing().set_certification()));
+    ///            Some(KeyFlags::empty().set_signing()));
     /// # Ok(()) }
     /// ```
     pub fn set_primary_key_flags(mut self, flags: KeyFlags) -> Self {
@@ -1185,8 +1230,14 @@ impl CertBuilder<'_> {
     /// after the validity period, the certificate is considered to be
     /// expired.
     ///
-    /// A value of None means that the certificate never expires.
-    //
+    /// The validity period starts with the creation time (see
+    /// [`CertBuilder::set_creation_time`]).
+    ///
+    /// A value of `None` means that the certificate never expires.
+    ///
+    /// See [this section](CertBuilder#expiration) of the type's
+    /// documentation for security considerations of key expiration.
+    ///
     /// # Examples
     ///
     /// ```
@@ -1279,7 +1330,7 @@ impl CertBuilder<'_> {
     ///         .generate()?;
     /// # Ok(()) }
     /// ```
-    pub fn generate(mut self) -> Result<(Cert, Signature)> {
+    pub fn generate(self) -> Result<(Cert, Signature)> {
         use crate::Packet;
         use crate::types::ReasonForRevocation;
         use std::convert::TryFrom;
@@ -1287,15 +1338,10 @@ impl CertBuilder<'_> {
         let creation_time =
             self.creation_time.unwrap_or_else(|| {
                 use crate::packet::signature::SIG_BACKDATE_BY;
-                time::SystemTime::now() -
+                crate::now() -
                     time::Duration::new(SIG_BACKDATE_BY, 0)
             });
 
-        // make sure the primary key can sign subkeys
-        if !self.subkeys.is_empty() {
-            self.primary.flags = self.primary.flags.set_certification();
-        }
-
         // Generate & self-sign primary key.
         let (primary, sig, mut signer) = self.primary_key(creation_time)?;
 
@@ -1576,14 +1622,14 @@ mod tests {
     }
 
     #[test]
-    fn always_certify() {
+    fn not_always_certify() {
         let p = &P::new();
         let (cert1, _) = CertBuilder::new()
             .set_cipher_suite(CipherSuite::Cv25519)
             .set_primary_key_flags(KeyFlags::empty())
             .add_transport_encryption_subkey()
             .generate().unwrap();
-        assert!(cert1.primary_key().with_policy(p, None).unwrap().for_certification());
+        assert!(! cert1.primary_key().with_policy(p, None).unwrap().for_certification());
         assert_eq!(cert1.keys().subkeys().count(), 1);
     }
 
@@ -1659,7 +1705,7 @@ mod tests {
     fn validity_periods() {
         let p = &P::new();
 
-        let now = std::time::SystemTime::now();
+        let now = crate::now();
         let s = std::time::Duration::new(1, 0);
 
         let (cert,_) = CertBuilder::new()
