@@ -22,8 +22,6 @@ use std::borrow::Cow;
 
 #[cfg(test)]
 use quickcheck::{Arbitrary, Gen};
-#[cfg(test)]
-use rand::Rng;
 
 use crate::types::{
     Curve,
@@ -159,7 +157,8 @@ impl MPI {
     ///
     /// Returns `Error::UnsupportedEllipticCurve` if the curve is not
     /// supported, `Error::MalformedMPI` if the point is formatted
-    /// incorrectly.
+    /// incorrectly, `Error::InvalidOperation` if the given curve is
+    /// operating on native octet strings.
     pub fn decode_point(&self, curve: &Curve) -> Result<(&[u8], &[u8])> {
         Self::decode_point_common(self.value(), curve)
     }
@@ -192,8 +191,13 @@ impl MPI {
                 Ok((&value[1..], &[]))
             },
 
-            _ => {
-
+            NistP256
+                | NistP384
+                | NistP521
+                | BrainpoolP256
+                | BrainpoolP512
+                =>
+            {
                 // Length of one coordinate in bytes, rounded up.
                 let coordinate_length = (curve.len()? + 7) / 8;
 
@@ -218,6 +222,9 @@ impl MPI {
                 Ok((&value[1..1 + coordinate_length],
                     &value[1 + coordinate_length..]))
             },
+
+            Unknown(_) =>
+                Err(Error::UnsupportedEllipticCurve(curve.clone()).into()),
         }
     }
 
@@ -259,7 +266,7 @@ impl Hash for MPI {
 
 #[cfg(test)]
 impl Arbitrary for MPI {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         loop {
             let buf = <Vec<u8>>::arbitrary(g);
 
@@ -425,7 +432,8 @@ impl ProtectedMPI {
     ///
     /// Returns `Error::UnsupportedEllipticCurve` if the curve is not
     /// supported, `Error::MalformedMPI` if the point is formatted
-    /// incorrectly.
+    /// incorrectly, `Error::InvalidOperation` if the given curve is
+    /// operating on native octet strings.
     pub fn decode_point(&self, curve: &Curve) -> Result<(&[u8], &[u8])> {
         MPI::decode_point_common(self.value(), curve)
     }
@@ -580,9 +588,11 @@ impl Hash for PublicKey {
 
 #[cfg(test)]
 impl Arbitrary for PublicKey {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         use self::PublicKey::*;
-        match g.gen_range(0, 6) {
+        use crate::arbitrary_helper::gen_arbitrary_from_range;
+
+        match gen_arbitrary_from_range(0..6, g) {
             0 => RSA {
                 e: MPI::arbitrary(g),
                 n: MPI::arbitrary(g),
@@ -842,8 +852,10 @@ impl Hash for SecretKeyMaterial {
 
 #[cfg(test)]
 impl Arbitrary for SecretKeyMaterial {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
-        match g.gen_range(0, 6) {
+    fn arbitrary(g: &mut Gen) -> Self {
+        use crate::arbitrary_helper::gen_arbitrary_from_range;
+
+        match gen_arbitrary_from_range(0..6, g) {
             0 => SecretKeyMaterial::RSA {
                 d: MPI::arbitrary(g).into(),
                 p: MPI::arbitrary(g).into(),
@@ -969,8 +981,10 @@ impl Hash for Ciphertext {
 
 #[cfg(test)]
 impl Arbitrary for Ciphertext {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
-        match g.gen_range(0, 3) {
+    fn arbitrary(g: &mut Gen) -> Self {
+        use crate::arbitrary_helper::gen_arbitrary_from_range;
+
+        match gen_arbitrary_from_range(0..3, g) {
             0 => Ciphertext::RSA {
                 c: MPI::arbitrary(g),
             },
@@ -1062,8 +1076,10 @@ impl Hash for Signature {
 
 #[cfg(test)]
 impl Arbitrary for Signature {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
-        match g.gen_range(0, 4) {
+    fn arbitrary(g: &mut Gen) -> Self {
+        use crate::arbitrary_helper::gen_arbitrary_from_range;
+
+        match gen_arbitrary_from_range(0..4, g) {
             0 => Signature::RSA  {
                 s: MPI::arbitrary(g),
             },
