@@ -508,9 +508,10 @@ impl fmt::Debug for Packet {
 
 #[cfg(test)]
 impl Arbitrary for Packet {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
-        use rand::Rng;
-        match g.gen_range(0, 15) {
+    fn arbitrary(g: &mut Gen) -> Self {
+        use crate::arbitrary_helper::gen_arbitrary_from_range;
+
+        match gen_arbitrary_from_range(0..15, g) {
             0 => Signature::arbitrary(g).into(),
             1 => OnePassSig::arbitrary(g).into(),
             2 => Key::<key::PublicParts, key::UnspecifiedRole>::arbitrary(g)
@@ -553,7 +554,7 @@ impl Arbitrary for Packet {
 }
 
 /// Fields used by multiple packet types.
-#[derive(Debug, Clone)]
+#[derive(Default, Debug, Clone)]
 pub struct Common {
     // In the future, this structure will hold the parsed CTB, packet
     // length, and lengths of chunks of partial body encoded packets.
@@ -573,20 +574,12 @@ assert_send_and_sync!(Common);
 
 #[cfg(test)]
 impl Arbitrary for Common {
-    fn arbitrary<G: Gen>(_: &mut G) -> Self {
+    fn arbitrary(_: &mut Gen) -> Self {
         // XXX: Change if this gets interesting fields.
         Common::default()
     }
 }
 
-impl Default for Common {
-    fn default() -> Common {
-        Common {
-            dummy: Default::default(),
-        }
-    }
-}
-
 impl PartialEq for Common {
     fn eq(&self, _: &Common) -> bool {
         // Don't compare anything.
@@ -825,7 +818,7 @@ fn packet_path_iter() {
             v.push(i);
             lpaths.push(v);
 
-            if let Some(ref container) = packet.container_ref() {
+            if let Some(container) = packet.container_ref() {
                 if let Some(c) = container.children() {
                     for mut path in paths(c).into_iter()
                     {
@@ -2017,4 +2010,19 @@ mod test {
             }
         }
     }
+
+    /// Problem on systems with 32-bit time_t.
+    #[test]
+    fn issue_802() -> Result<()> {
+        let pp = crate::PacketPile::from_bytes(b"-----BEGIN PGP ARMORED FILE-----
+
+xiEE/////xIJKyQDAwIIAQENAFYp8M2JngCfc04tIwMBCuU=
+-----END PGP ARMORED FILE-----
+")?;
+        let p = pp.path_ref(&[0]).unwrap();
+        let buf = p.to_vec().expect("Failed to serialize packet");
+        let q = Packet::from_bytes(&buf).unwrap();
+        assert_eq!(p, &q);
+        Ok(())
+    }
 }
