@@ -38,7 +38,7 @@ impl SKESK {
 
 #[cfg(test)]
 impl Arbitrary for SKESK {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         if bool::arbitrary(g) {
             SKESK::V4(SKESK4::arbitrary(g))
         } else {
@@ -177,7 +177,7 @@ impl SKESK4 {
         // We need to prefix the cipher specifier to the session key.
         let mut psk: SessionKey = vec![0; 1 + session_key.len()].into();
         psk[0] = payload_algo.into();
-        psk[1..].copy_from_slice(&session_key);
+        psk[1..].copy_from_slice(session_key);
         let mut esk = vec![0u8; psk.len()];
 
         for (pt, ct) in psk[..].chunks(block_size)
@@ -252,14 +252,14 @@ impl SKESK4 {
     {
         let key = self.s2k.derive_key(password, self.sym_algo.key_size()?)?;
 
-        if let Some(ref esk) = self.esk()? {
+        if let Some(esk) = self.esk()? {
             // Use the derived key to decrypt the ESK. Unlike SEP &
             // SEIP we have to use plain CFB here.
             let blk_sz = self.sym_algo.block_size()?;
             let iv = vec![0u8; blk_sz];
             let mut dec  = self.sym_algo.make_decrypt_cfb(&key[..], iv)?;
             let mut plain: SessionKey = vec![0u8; esk.len()].into();
-            let cipher = &esk[..];
+            let cipher = esk;
 
             for (pl, ct)
                 in plain[..].chunks_mut(blk_sz).zip(cipher.chunks(blk_sz))
@@ -272,15 +272,7 @@ impl SKESK4 {
             Ok((sym, plain[1..].into()))
         } else {
             // No ESK, we return the derived key.
-
-            #[allow(deprecated)]
-            match self.s2k {
-                S2K::Simple{ .. } =>
-                    Err(Error::InvalidOperation(
-                        "SKESK4: Cannot use Simple S2K without ESK".into())
-                        .into()),
-                _ => Ok((self.sym_algo, key)),
-            }
+            Ok((self.sym_algo, key))
         }
     }
 }
@@ -299,7 +291,7 @@ impl From<SKESK4> for Packet {
 
 #[cfg(test)]
 impl Arbitrary for SKESK4 {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         SKESK4::new(SymmetricAlgorithm::arbitrary(g),
                     S2K::arbitrary(g),
                     Option::<Vec<u8>>::arbitrary(g).map(|v| v.into()))
@@ -474,7 +466,7 @@ impl SKESK5 {
 
         // We need to prefix the cipher specifier to the session key.
         let mut esk = vec![0u8; session_key.len()];
-        ctx.encrypt(&mut esk, &session_key);
+        ctx.encrypt(&mut esk, session_key);
 
         // Digest.
         let mut digest = vec![0u8; esk_aead.digest_size()?];
@@ -500,10 +492,10 @@ impl SKESK5 {
         let key = self.s2k().derive_key(password,
                                         self.symmetric_algo().key_size()?)?;
 
-        if let Some(ref esk) = self.esk()? {
+        if let Some(esk) = self.esk()? {
             // Use the derived key to decrypt the ESK.
             let mut cipher = self.aead_algo.context(
-                self.symmetric_algo(), &key, &self.aead_iv()?, CipherOp::Decrypt)?;
+                self.symmetric_algo(), &key, self.aead_iv()?, CipherOp::Decrypt)?;
 
             let ad = [0xc3, 5 /* Version.  */, self.symmetric_algo().into(),
                       self.aead_algo.into()];
@@ -579,7 +571,7 @@ impl From<SKESK5> for Packet {
 
 #[cfg(test)]
 impl Arbitrary for SKESK5 {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         let algo = AEADAlgorithm::EAX;  // The only one we dig.
         let mut iv = vec![0u8; algo.iv_size().unwrap()];
         for b in iv.iter_mut() {
@@ -658,4 +650,61 @@ mod test {
         packets[0].serialize(&mut serialized).unwrap();
         assert_eq!(&raw[..], &serialized[..]);
     }
+
+    /// Tests various S2K methods, with and without encrypted session
+    /// key.
+    #[test]
+    fn skesk4_s2k_variants() -> Result<()> {
+        use std::io::Read;
+        use crate::{
+            Cert,
+            Fingerprint,
+            packet::{SKESK, PKESK},
+            parse::stream::*,
+        };
+
+        struct H();
+        impl VerificationHelper for H {
+            fn get_certs(&mut self, _ids: &[crate::KeyHandle])
+                         -> Result<Vec<Cert>> {
+                Ok(Vec::new())
+            }
+
+            fn check(&mut self, _m: MessageStructure)
+                     -> Result<()> {
+                Ok(())
+            }
+        }
+
+        impl DecryptionHelper for H {
+            fn decrypt<D>(&mut self, _: &[PKESK], skesks: &[SKESK],
+                          _: Option<SymmetricAlgorithm>,
+                          mut decrypt: D) -> Result<Option<Fingerprint>>
+            where D: FnMut(SymmetricAlgorithm, &SessionKey) -> bool
+            {
+                assert_eq!(skesks.len(), 1);
+                let (cipher, sk) = skesks[0].decrypt(&"password".into())?;
+                assert_eq!(cipher, SymmetricAlgorithm::AES256);
+                let r = decrypt(cipher, &sk);
+                assert!(r);
+                Ok(None)
+            }
+        }
+
+        let p = &crate::policy::StandardPolicy::new();
+        for variant in &["simple", "salted", "iterated.min", "iterated.max"] {
+            for esk in &["", ".esk"] {
+                let name = format!("s2k/{}{}.pgp", variant, esk);
+                eprintln!("{}", name);
+                let mut verifier = DecryptorBuilder::from_bytes(
+                    crate::tests::message(&name))?
+                    .with_policy(p, None, H())?;
+                let mut b = Vec::new();
+                verifier.read_to_end(&mut b)?;
+                assert_eq!(&b, b"Hello World :)");
+            }
+        }
+
+        Ok(())
+    }
 }
