@@ -263,27 +263,20 @@ impl fmt::Display for PublicKeyAlgorithm {
 
 #[cfg(test)]
 impl Arbitrary for PublicKeyAlgorithm {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         u8::arbitrary(g).into()
     }
 }
 
 #[cfg(test)]
 impl PublicKeyAlgorithm {
-    pub(crate) fn arbitrary_for_signing<G: Gen>(g: &mut G) -> Self {
-        use rand::Rng;
+    pub(crate) fn arbitrary_for_signing(g: &mut Gen) -> Self {
         use self::PublicKeyAlgorithm::*;
+
         #[allow(deprecated)]
-        let a = match g.gen_range(0, 5) {
-            0 => RSAEncryptSign,
-            1 => RSASign,
-            2 => DSA,
-            3 => ECDSA,
-            4 => EdDSA,
-            _ => unreachable!(),
-        };
+        let a = g.choose(&[RSAEncryptSign, RSASign, DSA, ECDSA, EdDSA]).unwrap();
         assert!(a.for_signing());
-        a
+        *a
     }
 }
 
@@ -489,7 +482,7 @@ impl Curve {
 
 #[cfg(test)]
 impl Arbitrary for Curve {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         match u8::arbitrary(g) % 8 {
             0 => Curve::NistP256,
             1 => Curve::NistP384,
@@ -658,7 +651,7 @@ impl fmt::Display for SymmetricAlgorithm {
 
 #[cfg(test)]
 impl Arbitrary for SymmetricAlgorithm {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         u8::arbitrary(g).into()
     }
 }
@@ -719,9 +712,7 @@ impl AEADAlgorithm {
     /// use sequoia_openpgp as openpgp;
     /// use openpgp::types::AEADAlgorithm;
     ///
-    /// assert!(AEADAlgorithm::EAX.is_supported());
-    ///
-    /// assert!(!AEADAlgorithm::OCB.is_supported());
+    /// assert!(! AEADAlgorithm::Private(100).is_supported());
     /// ```
     pub fn is_supported(&self) -> bool {
         self.is_supported_by_backend()
@@ -767,7 +758,7 @@ impl fmt::Display for AEADAlgorithm {
 
 #[cfg(test)]
 impl Arbitrary for AEADAlgorithm {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         u8::arbitrary(g).into()
     }
 }
@@ -849,7 +840,6 @@ impl CompressionAlgorithm {
     /// use openpgp::types::CompressionAlgorithm;
     ///
     /// assert!(CompressionAlgorithm::Uncompressed.is_supported());
-    /// assert!(CompressionAlgorithm::Zip.is_supported());
     ///
     /// assert!(!CompressionAlgorithm::Private(101).is_supported());
     /// ```
@@ -909,7 +899,7 @@ impl fmt::Display for CompressionAlgorithm {
 
 #[cfg(test)]
 impl Arbitrary for CompressionAlgorithm {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         u8::arbitrary(g).into()
     }
 }
@@ -1079,7 +1069,7 @@ impl HashAlgorithm {
 
 #[cfg(test)]
 impl Arbitrary for HashAlgorithm {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         u8::arbitrary(g).into()
     }
 }
@@ -1250,7 +1240,7 @@ impl fmt::Display for SignatureType {
 
 #[cfg(test)]
 impl Arbitrary for SignatureType {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         u8::arbitrary(g).into()
     }
 }
@@ -1394,7 +1384,7 @@ impl fmt::Display for ReasonForRevocation {
 
 #[cfg(test)]
 impl Arbitrary for ReasonForRevocation {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         u8::arbitrary(g).into()
     }
 }
@@ -1641,7 +1631,7 @@ impl fmt::Display for DataFormat {
 
 #[cfg(test)]
 impl Arbitrary for DataFormat {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         u8::arbitrary(g).into()
     }
 }
@@ -1712,7 +1702,7 @@ mod tests {
 
     quickcheck! {
         fn comp_roundtrip(comp: CompressionAlgorithm) -> bool {
-            let val: u8 = comp.clone().into();
+            let val: u8 = comp.into();
             comp == CompressionAlgorithm::from(val)
         }
     }
@@ -1728,7 +1718,7 @@ mod tests {
         fn comp_parse(comp: CompressionAlgorithm) -> bool {
             match comp {
                 CompressionAlgorithm::Unknown(u) => u > 110 || (u > 3 && u < 100),
-                CompressionAlgorithm::Private(u) => u >= 100 && u <= 110,
+                CompressionAlgorithm::Private(u) => (100..=110).contains(&u),
                 _ => true
             }
         }
@@ -1737,7 +1727,7 @@ mod tests {
 
     quickcheck! {
         fn sym_roundtrip(sym: SymmetricAlgorithm) -> bool {
-            let val: u8 = sym.clone().into();
+            let val: u8 = sym.into();
             sym == SymmetricAlgorithm::from(val)
         }
     }
@@ -1755,7 +1745,7 @@ mod tests {
                 SymmetricAlgorithm::Unknown(u) =>
                     u == 5 || u == 6 || u > 110 || (u > 10 && u < 100),
                 SymmetricAlgorithm::Private(u) =>
-                    u >= 100 && u <= 110,
+                    (100..=110).contains(&u),
                 _ => true
             }
         }
@@ -1764,7 +1754,7 @@ mod tests {
 
     quickcheck! {
         fn aead_roundtrip(aead: AEADAlgorithm) -> bool {
-            let val: u8 = aead.clone().into();
+            let val: u8 = aead.into();
             aead == AEADAlgorithm::from(val)
         }
     }
@@ -1782,7 +1772,7 @@ mod tests {
                 AEADAlgorithm::Unknown(u) =>
                     u == 0 || u > 110 || (u > 2 && u < 100),
                 AEADAlgorithm::Private(u) =>
-                    u >= 100 && u <= 110,
+                    (100..=110).contains(&u),
                 _ => true
             }
         }
@@ -1791,7 +1781,7 @@ mod tests {
 
     quickcheck! {
         fn pk_roundtrip(pk: PublicKeyAlgorithm) -> bool {
-            let val: u8 = pk.clone().into();
+            let val: u8 = pk.into();
             pk == PublicKeyAlgorithm::from(val)
         }
     }
@@ -1807,9 +1797,9 @@ mod tests {
         fn pk_parse(pk: PublicKeyAlgorithm) -> bool {
             match pk {
                 PublicKeyAlgorithm::Unknown(u) =>
-                    u == 0 || u > 110 || (u >= 4 && u <= 15)
-                    || (u >= 18 && u < 100),
-                PublicKeyAlgorithm::Private(u) => u >= 100 && u <= 110,
+                    u == 0 || u > 110 || (4..=15).contains(&u)
+                    || (18..100).contains(&u),
+                PublicKeyAlgorithm::Private(u) => (100..=110).contains(&u),
                 _ => true
             }
         }
@@ -1825,7 +1815,7 @@ mod tests {
 
     quickcheck! {
         fn signature_type_roundtrip(t: SignatureType) -> bool {
-            let val: u8 = t.clone().into();
+            let val: u8 = t.into();
             t == SignatureType::from(val)
         }
     }
@@ -1840,7 +1830,7 @@ mod tests {
 
     quickcheck! {
         fn hash_roundtrip(hash: HashAlgorithm) -> bool {
-            let val: u8 = hash.clone().into();
+            let val: u8 = hash.into();
             hash == HashAlgorithm::from(val)
         }
     }
@@ -1880,8 +1870,8 @@ mod tests {
         fn hash_parse(hash: HashAlgorithm) -> bool {
             match hash {
                 HashAlgorithm::Unknown(u) => u == 0 || (u > 11 && u < 100) ||
-                    u > 110 || (u >= 4 && u <= 7) || u == 0,
-                HashAlgorithm::Private(u) => u >= 100 && u <= 110,
+                    u > 110 || (4..=7).contains(&u) || u == 0,
+                HashAlgorithm::Private(u) => (100..=110).contains(&u),
                 _ => true
             }
         }
@@ -1889,7 +1879,7 @@ mod tests {
 
     quickcheck! {
         fn rfr_roundtrip(rfr: ReasonForRevocation) -> bool {
-            let val: u8 = rfr.clone().into();
+            let val: u8 = rfr.into();
             rfr == ReasonForRevocation::from(val)
         }
     }
@@ -1909,7 +1899,7 @@ mod tests {
                     || (u > 32 && u < 100)
                     || u > 110,
                 ReasonForRevocation::Private(u) =>
-                    u >= 100 && u <= 110,
+                    (100..=110).contains(&u),
                 _ => true
             }
         }
@@ -1917,7 +1907,7 @@ mod tests {
 
     quickcheck! {
         fn df_roundtrip(df: DataFormat) -> bool {
-            let val: u8 = df.clone().into();
+            let val: u8 = df.into();
             df == DataFormat::from(val)
         }
     }
